/* packet-ct_sync.c
 * Routines for ct_sync packet disassembly
 *
 * $Id: packet-ctsync.c 11615 2004-08-06 19:57:49Z guy $
 *
 * Copyright (c) 2004 by Harald Welte <laforge@netfilter.org>
 *
 * Ethereal - Network traffic analyzer
 * By Gerald Combs <gerald@ethereal.com>
 * Copyright 1999 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifndef HAVE_WIN32_LIBETHEREAL_LIB
#include "plugins/plugin_api.h"
#endif

#include "moduleinfo.h"

#include <stdio.h>
#include <stdlib.h>
#include <gmodule.h>
#include <ctype.h>
#include <time.h>
#include <string.h>
#include <epan/packet.h>
#include <epan/addr_resolv.h>
#include "prefs.h"
#include <epan/strutil.h>
#include <epan/conversation.h>
#include "packet-ctsync.h"
#include "../../tap.h"

#ifndef HAVE_WIN32_LIBETHEREAL_LIB
#include "plugins/plugin_api_defs.h"
#endif

#ifndef ENABLE_STATIC
G_MODULE_EXPORT const gchar version[] = VERSION;
#endif

void proto_reg_handoff_ctsync(void);


/* Define the ctsync proto */
static int proto_ctsync = -1;
static int proto_ctsync_msg = -1;
static int proto_conntrack_tuple = -1;

/* Define many many headers for ctsync */
static int hf_ctsync_version = -1;
static int hf_ctsync_type = -1;
static int hf_ctsync_nodeid = -1;
static int hf_ctsync_msgcount = -1;
static int hf_ctsync_seq = -1;
static int hf_ctsync_minseq = -1;
static int hf_ctsync_flags = -1;
static int hf_ctsync_flags_recover = -1;

static int hf_ctsync_pkt_mann_upsince = -1;
static int hf_ctsync_pkt_mann_flags = -1;

static const value_string ctsync_pkt_type_vals[] = {
	{0x00, "Unknown Packet"},
	{0x01, "Master ANNOUNCE"},
	{0x02, "Slave INITSYNC Request"},
	{0x03, "SYNC Packet"},
	{0x04, "NACK Packet"},
	{0x05, "INITSYNC Follows"},
};

/*
 * Define the trees for mgcp
 * We need one for MGCP itself, one for the MGCP paramters and one
 * for each of the dissected parameters
 */
static int ett_ctsync = -1;
static int ett_ctsync_flags = -1;
static int ett_ctsync_msg = -1;
static int ett_conntrack_tuple = -1;

/*
 * Define the tap for mgcp
 */
static int ctsync_tap = -1;

static gboolean ctsync_summary_in_tree = TRUE;


static int hf_conntrack_tuple_src_ip = -1;
static int hf_conntrack_tuple_src_u_all = -1;
static int hf_conntrack_tuple_src_u_port = -1;
static int hf_conntrack_tuple_dst_ip = -1;
static int hf_conntrack_tuple_dst_u_all = -1;
static int hf_conntrack_tuple_dst_u_port = -1;
static int hf_conntrack_tuple_dst_protonum = -1;

#define NIPQUAD(addr) \
        ((unsigned char *)&addr)[0], \
        ((unsigned char *)&addr)[1], \
        ((unsigned char *)&addr)[2], \
        ((unsigned char *)&addr)[3]


static void
dissect_conntrack_tuple(char *prefix, tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
{
	int offset = 0;
	proto_tree *conntrack_tuple_tree;
	proto_item *ti;

	if (ctsync_summary_in_tree) {
		guint32 sip;
		guint32 dip;

		tvb_memcpy(tvb, (guint8*)&sip, offset, 4);
		tvb_memcpy(tvb, (guint8*)&dip, offset+8, 4);

		ti = proto_tree_add_protocol_format(tree, proto_conntrack_tuple,
						    tvb, 0, -1,
			"%sip_conntrack_tuple: "
			"%u.%u.%u.%u:%u->%u.%u.%u.%u:%u", prefix,
			NIPQUAD(sip), tvb_get_ntohs(tvb, offset+4), 
			NIPQUAD(dip), tvb_get_ntohs(tvb, offset+12));
	} else
		ti = proto_tree_add_item(tree, proto_conntrack_tuple, 
					 tvb, 0, -1, FALSE);
	conntrack_tuple_tree = proto_item_add_subtree(ti, ett_conntrack_tuple);


	/* this is for a pptp-enlarged little-endian tuple */
	proto_tree_add_item(conntrack_tuple_tree, hf_conntrack_tuple_src_ip, 
			    tvb, offset, 4, TRUE);
	offset += 4; 

	proto_tree_add_item(conntrack_tuple_tree, hf_conntrack_tuple_src_u_all,
			    tvb, offset, 4, FALSE);
	proto_tree_add_item(conntrack_tuple_tree, hf_conntrack_tuple_src_u_port,
			    tvb, offset, 2, FALSE);
	offset += 4;

	proto_tree_add_item(conntrack_tuple_tree, hf_conntrack_tuple_dst_ip,
			    tvb, offset, 4, TRUE);

	offset += 4;

	proto_tree_add_item(conntrack_tuple_tree, hf_conntrack_tuple_dst_u_all,
			    tvb, offset, 4, FALSE);
	proto_tree_add_item(conntrack_tuple_tree, hf_conntrack_tuple_dst_u_port,
			    tvb, offset, 2, FALSE);
	offset += 8;

	proto_tree_add_item(conntrack_tuple_tree, 
			    hf_conntrack_tuple_dst_protonum, tvb,
			    offset, 2, TRUE);
	offset += 2;
}

#define CONNTRACK_TUPLE_SIZE		22
#define CONNTRACK_TUPLE_SIZE_ALIGNED	24

static void
dissect_ctsync_msg_conntrack_update(tvbuff_t *tvb, int offset, packet_info
				    *pinfo, proto_tree *tree)
{
	tvbuff_t *new_tvb;

	/* orig tuple */
	new_tvb = tvb_new_subset(tvb, offset, CONNTRACK_TUPLE_SIZE_ALIGNED, 
				 CONNTRACK_TUPLE_SIZE);
	dissect_conntrack_tuple("ORIGINAL ", new_tvb, pinfo, tree);
	offset += CONNTRACK_TUPLE_SIZE_ALIGNED;

	/* reply tuple */
	new_tvb = tvb_new_subset(tvb, offset, CONNTRACK_TUPLE_SIZE_ALIGNED, 
				 CONNTRACK_TUPLE_SIZE);
	dissect_conntrack_tuple("REPLY ", new_tvb, pinfo, tree);
	offset += CONNTRACK_TUPLE_SIZE_ALIGNED;

	/* status */
	proto_tree_add_text(tree, tvb, offset, 4,
			    "Status: 0x%08x", tvb_get_ntohl(tvb, offset));
	offset += 4;

	/* expires */
	proto_tree_add_text(tree, tvb, offset, 4,
			   "Expires: 0x%08x", tvb_get_ntohl(tvb, offset));
	offset += 4;

	/* helper */

	/* helper_info */

	/* expecting */

	/* master_ct_tuple */

	/* protocol_info */

	/* nat */

	/* mark */
}

static void
dissect_ctsync_msg_conntrack_delete(tvbuff_t *tvb, int offset,
				    packet_info *pinfo, proto_tree *tree)
{
	tvbuff_t *new_tvb;

	/* orig tuple */
	new_tvb = tvb_new_subset(tvb, offset, CONNTRACK_TUPLE_SIZE_ALIGNED,
				 CONNTRACK_TUPLE_SIZE);
	dissect_conntrack_tuple("ORIGINAL ", new_tvb, pinfo, tree);
	offset += CONNTRACK_TUPLE_SIZE_ALIGNED;
}

static void
dissect_ctsync_msg_expect_update(tvbuff_t *tvb, int offset,
				 packet_info *pinfo, proto_tree *tree)
{
	/* not yet implemented */
}

static void
dissect_ctsync_msg_expect_delete(tvbuff_t *tvb, int offset,
				 packet_info *pinfo, proto_tree *tree)
{
	/* not yet implemented */
}



static int hf_ctsync_msg_resource = -1;
static int hf_ctsync_msg_type = -1;
static int hf_ctsync_msg_len = -1;
static int hf_ctsync_msg_flags = -1;

static const value_string ctsync_msg_type_vals[] = {
	{0x01, "Update"},
	{0x02, "Delete"},
};

static const value_string ctsync_msg_resource_vals[] = {
	{0x01, "Connection Tracking Entry"},
	{0x02, "Connection Tracking Expectation"},
};

static const value_string ctsync_msg_flag_vals[] = {
	{0x00, "None"},
	{0x01, "New Entry"},
	{0x02, "Entry is part of Initsync"},
};

#define CTSYNC_PKTHDR_SIZE	12
#define CTSYNC_MSGHDR_SIZE	8

static void
dissect_ctsync_msg(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
{
	int offset = 0;
	struct ct_sync_msghdr cts_msg;
	tvbuff_t *next_tvb;
	proto_tree *ctsync_msg_tree;
	proto_item *ti;

	cts_msg.resource = tvb_get_guint8(tvb, offset);
	cts_msg.type = tvb_get_guint8(tvb, offset + 1);
	cts_msg.len = tvb_get_ntohs(tvb, offset + 2);
	cts_msg.flags = tvb_get_guint8(tvb, offset + 4);

	if (ctsync_summary_in_tree) {
		ti = proto_tree_add_protocol_format(tree, proto_ctsync_msg, 
						    tvb, 0, -1, 
			"ct_sync Message, Resource: %s, Type: %s",
			val_to_str(tvb_get_guint8(tvb, offset),
				   ctsync_msg_resource_vals, "%s"),
			val_to_str(tvb_get_guint8(tvb, offset+1),
				   ctsync_msg_type_vals, "%s"));
	} else
		ti = proto_tree_add_item(tree, proto_ctsync_msg, tvb, 0, -1, FALSE);
	ctsync_msg_tree = proto_item_add_subtree(ti, ett_ctsync_msg);

	proto_tree_add_item(ctsync_msg_tree, hf_ctsync_msg_resource, tvb, 
			    offset, 1, FALSE);
	proto_tree_add_item(ctsync_msg_tree, hf_ctsync_msg_type, tvb, 
			    offset+1, 1, FALSE);
	proto_tree_add_item(ctsync_msg_tree, hf_ctsync_msg_len, tvb,
			    offset+2, 2, FALSE);
	proto_tree_add_item(ctsync_msg_tree, hf_ctsync_msg_flags, tvb,
			    offset+4, 1, FALSE);

	next_tvb = tvb_new_subset(tvb, offset, cts_msg.len, cts_msg.len);
	//dissector_next(next_tvb, pinfo, tree);
	
	switch (cts_msg.resource) {
	case 0x01:
		switch (cts_msg.type) {
		case 0x01:
			dissect_ctsync_msg_conntrack_update(tvb, CTSYNC_MSGHDR_SIZE, pinfo, ctsync_msg_tree);
			break;
		case 0x02:
			dissect_ctsync_msg_conntrack_delete(tvb, CTSYNC_MSGHDR_SIZE, pinfo, ctsync_msg_tree);
			break;
		}
		break;
	}
}

/*
 * dissect_ctsync - The dissector for the CT_SYNC Protocol
 */
static void
dissect_ctsync_pkt_master_ann(tvbuff_t *tvb, int offset, 
			      packet_info *pinfo, proto_tree *tree)
{
	proto_tree_add_item(tree, hf_ctsync_pkt_mann_upsince,
			    tvb, offset, 4, FALSE);
	proto_tree_add_item(tree, hf_ctsync_pkt_mann_flags,
			    tvb, offset, 1, FALSE);
}

static void
dissect_ctsync(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree)
{
	int		offset = 0;
	struct		ct_sync_pkthdr cts_pkt;
	proto_tree 	*ctsync_tree = NULL;
	proto_item 	*ti = NULL;
	tvbuff_t 	*next_tvb;

	int 		msg_len = 0;
	int		msg_hdr_offset;

	/* Set the columns now, so that they'll be set correctly if we throw an
	 * exception.  We can set them later as well....  */
	if (check_col(pinfo->cinfo, COL_PROTOCOL))
		col_add_str(pinfo->cinfo, COL_PROTOCOL, "ctsync");
	if (check_col(pinfo->cinfo, COL_INFO))
		col_clear(pinfo->cinfo, COL_INFO);

	cts_pkt.version = tvb_get_guint8(tvb, offset + 0);
	cts_pkt.pkttype = tvb_get_guint8(tvb, offset + 1);

	switch (cts_pkt.version) {
	case 1:
		cts_pkt.pktseq = tvb_get_ntohs(tvb, offset + 2);
		cts_pkt.node_id = tvb_get_guint8(tvb, offset + 4);
		cts_pkt.count = tvb_get_guint8(tvb, offset + 5);
		cts_pkt.minseq = tvb_get_ntohs(tvb, offset + 6);
		break;
	case 2:
		cts_pkt.node_id = tvb_get_guint8(tvb, offset + 2);
		cts_pkt.count = tvb_get_guint8(tvb, offset + 3);
		cts_pkt.pktseq = tvb_get_ntohs(tvb, offset + 4);
		cts_pkt.minseq = tvb_get_ntohs(tvb, offset + 6);
		cts_pkt.flags = tvb_get_guint8(tvb, offset + 8);
		break;
	}
	if (check_col(pinfo->cinfo, COL_INFO)) {
		col_add_fstr(pinfo->cinfo, COL_INFO, 
			     "Version: %u, Type: %s, Count: %u",
			     cts_pkt.version,
			     val_to_str(cts_pkt.pkttype, 
				        ctsync_pkt_type_vals, "%s"),
			     cts_pkt.count);
	}
	if (tree) {
		if (ctsync_summary_in_tree) {
			ti = proto_tree_add_protocol_format(tree, proto_ctsync,
							    tvb, 0, -1, 
				"Connection Tracking Sync Protocol, "
				"Version: %u, Type: %s",
				 cts_pkt.version,
				 val_to_str(cts_pkt.pkttype, 
					    ctsync_pkt_type_vals, "%s")
				);
		} else {
			ti = proto_tree_add_item(tree, proto_ctsync, tvb, 0,
						 -1, FALSE);
		}
		ctsync_tree = proto_item_add_subtree(ti, ett_ctsync);

		proto_tree_add_item(ctsync_tree, hf_ctsync_version,
			    	    tvb, offset, 1, FALSE);
		proto_tree_add_item(ctsync_tree, hf_ctsync_type,
				    tvb, offset+1, 1, FALSE);
		switch (cts_pkt.version) {
		case 1:
			proto_tree_add_item(ctsync_tree, hf_ctsync_seq,
					    tvb, offset+2, 2, FALSE);
			proto_tree_add_item(ctsync_tree, hf_ctsync_nodeid,
					    tvb, offset+4, 1, FALSE);
			proto_tree_add_item(ctsync_tree, hf_ctsync_msgcount,
					    tvb, offset+5, 1, FALSE);
			proto_tree_add_item(ctsync_tree, hf_ctsync_minseq,
					    tvb, offset+6, 2, FALSE);
			break;
		case 2:
			proto_tree_add_item(ctsync_tree, hf_ctsync_nodeid,
					    tvb, offset+2, 1, FALSE);
			proto_tree_add_item(ctsync_tree, hf_ctsync_msgcount,
					    tvb, offset+3, 1, FALSE);
			proto_tree_add_item(ctsync_tree, hf_ctsync_seq,
					    tvb, offset+4, 2, FALSE);
			proto_tree_add_item(ctsync_tree, hf_ctsync_minseq,
					    tvb, offset+4, 2, FALSE);
			proto_tree_add_item(ctsync_tree, hf_ctsync_flags,
					    tvb, offset+8, 1, FALSE);
			break;
		}
	}

	switch (cts_pkt.pkttype) {
	case 0x01:
		/* Master Announce */
		dissect_ctsync_pkt_master_ann(tvb, offset + CTSYNC_PKTHDR_SIZE,
					      pinfo, ctsync_tree);
		break;
	case 0x02:
		/* Initsync Request */
		break;
	case 0x03:
		/* SYNC Packet */
		for (msg_hdr_offset = offset + CTSYNC_PKTHDR_SIZE;
		     tvb_length_remaining(tvb, msg_hdr_offset) 
		     				>= CTSYNC_MSGHDR_SIZE;
		     msg_hdr_offset += msg_len + CTSYNC_MSGHDR_SIZE) {
	
		        msg_len = tvb_get_ntohs(tvb, msg_hdr_offset+2);
			next_tvb = tvb_new_subset(tvb, msg_hdr_offset, 
						  msg_len+CTSYNC_MSGHDR_SIZE, 
						  -1);
			dissect_ctsync_msg(next_tvb, pinfo, ctsync_tree);
		}
		break;
	case 0x04:
		/* NACK Packet */
		break;
	}
	
}

/* Discard and init any state we've saved */

static void
ctsync_init_protocol(void)
{
}

/* Register all the bits needed with the filtering engine */

void
proto_register_ctsync(void)
{
	static hf_register_info hf[] = {
		/* Packet Header */
		{ &hf_ctsync_version,
		  { "Version", "ctsync.version", FT_UINT8, BASE_DEC, NULL, 0x0,
		    "Protocol Version", HFILL } 
		},
		{ &hf_ctsync_type,
		  { "Type", "ctsync.type", FT_UINT8, BASE_HEX, 
		    VALS(ctsync_pkt_type_vals), 0x0, "Packet type", HFILL }
		},
    		{ &hf_ctsync_nodeid,
		  { "Node ID", "ctsync.nodeid", FT_UINT8, BASE_DEC, NULL, 0,
		    "Node ID of Sender", HFILL }
		},
		{ &hf_ctsync_msgcount,
		  { "Message Count", "ctsync.msgcount", FT_UINT8, BASE_DEC,
		    NULL, 0, "Number of Messages in this Packet", HFILL }
		},
		{ &hf_ctsync_seq,
		  { "Sequence Number", "ctsync.seq", FT_UINT16, BASE_DEC, NULL,
		    0, "Sequence Number", HFILL }
		},
		{ &hf_ctsync_minseq, 
		  { "Minimum Sequence", "ctsync.minseq", FT_UINT16, BASE_DEC,
		    NULL, 0, "Minimum Sequence Number at Sender", HFILL }
		},
		{ &hf_ctsync_flags, 
		  { "Flags", "ctsync.flags", FT_UINT8, BASE_HEX, NULL, 0,
		    "Flags", HFILL }
		},
		{ &hf_ctsync_flags_recover,
		  { "Recover Flag", "ctsync.flags.recover", FT_BOOLEAN,
		    BASE_NONE, NULL, 0, 
		    "Flag indicating Packet is part of Recovery", HFILL }
		},
		/* Master Announce */
		{ &hf_ctsync_pkt_mann_upsince,
		  { "Uptime since", "ctsync.master_announce.upsince", FT_UINT32,
		    BASE_DEC, NULL, 0, "Uptime of announcing master", HFILL }
		},
		{ &hf_ctsync_pkt_mann_flags,
		  { "Flags", "ctsync.master_announce.flags", FT_UINT32,
		    BASE_DEC, NULL, 0, "Flags of Master Announcement", HFILL }
		},
	};

	static hf_register_info hf_msg[] = {
		/* Message Header */
		{ &hf_ctsync_msg_resource,
		  { "Resource", "ctsync.msg.resource", FT_UINT8, BASE_DEC,
		    VALS(ctsync_msg_resource_vals), 0, 
		    "Synchronization Ressource", HFILL },
		},
		{ &hf_ctsync_msg_type,
		  { "Message Type", "ctsync.msg.type", FT_UINT8, BASE_DEC,
		    VALS(ctsync_msg_type_vals), 0,
		    "Message Type (Operation)", HFILL },
		},
		{ &hf_ctsync_msg_len, 
		  { "Message Length", "ctsync.msg.len", FT_UINT16, BASE_DEC,
		    NULL, 0, "Message Length", HFILL },
		},
		{ &hf_ctsync_msg_flags, 
		  { "Message Flags", "ctsync.msg.flags", FT_UINT8, BASE_HEX,
		    VALS(ctsync_msg_flag_vals), 0,
		    "Message Flags", HFILL },
		},
		/* Add more fields here */
	};

	static hf_register_info hf_conntrack_tuple[] = {
		/* Message Header */
		{ &hf_conntrack_tuple_src_ip,
		  { "Source IP", "conntrack_tuple.src.ip", FT_IPv4, BASE_NONE,
		    NULL, 0, "Source IPv4 address of tuple", HFILL },
		},
		{ &hf_conntrack_tuple_src_u_all,
		  { "Source L4", "conntrack_tuple.src.u.all", FT_UINT32, 
		    BASE_DEC, NULL, 0, "Source L4 Part of Tuple", HFILL },
		},
		{ &hf_conntrack_tuple_src_u_port,
		  { "Source Port", "conntrack_tuple.dst.u.port", 
		    FT_UINT16, BASE_DEC, NULL, 0, "Source Port", HFILL},
		},
		{ &hf_conntrack_tuple_dst_ip,
		  { "Destination IP", "conntrack_tuple.dst.ip", FT_IPv4,
		    BASE_NONE, NULL, 0, "Source IPv4 Address of Tuple", HFILL },
		},
		{ &hf_conntrack_tuple_dst_u_all,
		  { "Destination L4", "conntrack_tuple.dst.u.all", FT_UINT32, 
		    BASE_DEC, NULL, 0, "Destination L4 Part of Tuple", HFILL },
		},
		{ &hf_conntrack_tuple_dst_u_port,
		  { "Destination Port", "conntrack_tuple.dst.u.port", 
		    FT_UINT16, BASE_DEC, NULL, 0, "Destination Port", HFILL},
		},
		{ &hf_conntrack_tuple_dst_protonum,
		  { "Layer 4 Protocol", "conntrack_tuple.dst.protonum", 
		    FT_UINT32, BASE_DEC, NULL, 0, 
		    "Source L4 Part of Tuple", HFILL },
		},
		/* Add more fields here */
	};

	static gint *ett[] = {
		&ett_ctsync,
		&ett_ctsync_flags,
  	};
	static gint *ett_msg[] = {
		&ett_ctsync_msg,
	};
	static gint *ett_conntrack[] = {
		&ett_conntrack_tuple,
	};
  	module_t *ctsync_module;

  	proto_ctsync = proto_register_protocol("netfilter ct_sync Protocol",
				       		"ctsync", "ctsync");
	proto_register_field_array(proto_ctsync, hf, array_length(hf));
	proto_register_subtree_array(ett, array_length(ett));
	register_init_routine(&ctsync_init_protocol);

  	proto_ctsync_msg = proto_register_protocol("netfilter ct_sync Message",
				       		"ctsync_msg", "ctsync_msg");
	proto_register_subtree_array(ett_msg, array_length(ett_msg));
	proto_register_field_array(proto_ctsync_msg, hf_msg,
				   array_length(hf_msg));

	proto_conntrack_tuple = proto_register_protocol(
						"netfilter conntrack tuple",
				       		"conntrack_tuple", 
						"conntrack_tuple");
	proto_register_subtree_array(ett_conntrack, 
				     array_length(ett_conntrack));
	proto_register_field_array(proto_conntrack_tuple, hf_conntrack_tuple,
				   array_length(hf_conntrack_tuple));


	/* Register our configuration options for , particularly our ports */

	ctsync_module = prefs_register_protocol(proto_ctsync,
					      proto_reg_handoff_ctsync);
	prefs_register_bool_preference(ctsync_module, "summary_in_tree",
		"Show ct_sync summary in protocol tree",
		"Whether the ct_sync summary line should be shown in the "
		"protocol tree", &ctsync_summary_in_tree);

	ctsync_tap = register_tap("ctsync");
}

/* The registration hand-off routine */
void
proto_reg_handoff_ctsync(void)
{
	static int mgcp_prefs_initialized = FALSE;
	static dissector_handle_t ctsync_handle;

	ctsync_handle = create_dissector_handle(dissect_ctsync, proto_ctsync);
	dissector_add("udp.port", UDP_PORT_CTSYNC, ctsync_handle);
}

/* Start the functions we need for the plugin stuff */

#ifndef ENABLE_STATIC

G_MODULE_EXPORT void
plugin_reg_handoff(void){
	proto_reg_handoff_ctsync();
}

#ifndef HAVE_WIN32_LIBETHEREAL_LIB

G_MODULE_EXPORT void
plugin_init(plugin_address_table_t *pat
#ifndef PLUGINS_NEED_ADDRESS_TABLE
_U_
#endif
){
	/* initialise the table of pointers needed in Win32 DLLs */
	plugin_address_table_init(pat);
  
#else /* HAVE_WIN32_LIBETHEREAL_LIB */
G_MODULE_EXPORT void plugin_init(void *dummy _U_)
{
#endif
	/* register the new protocol, protocol fields, and subtrees */
	if (proto_ctsync == -1) { /* execute protocol init only once */
		proto_register_ctsync();
	}
}

#endif

/* End the functions we need for plugin stuff */
